<?php
/**
 * Created by PhpStorm.
 * User: tuliomagalhaes
 * Date: 20/06/17
 * Time: 10:46
 */

namespace Service\ACL;


use Doctrine\ORM\EntityManager;
use Util\Utilities;
use \Zeedhi\ZhuAcl\Service\AccessControlRule as AccessControlRuleZeedhi;
use \Zeedhi\ZhuAcl\Util\TableProperties as ZhuAclTableProperties;

class AccessProfile {


    /**
     * @var \Factory\CpPerfacessoper
     */
    private $cpPerfacessoperFactory;
    /**
     * @var \Factory\CpPerfilacesso
     */
    private $cpPerfilacessoFactory;

    /** @var \Teknisa\Model\Teknisa\Repositories\CpPerfacessoper  */
    protected $cpPerfacessoperRepository;

    /** @var \Teknisa\Model\Teknisa\Repositories\CpPerfilacesso  */
    protected $cpPerfilacessorRepository;

    /** @var \Teknisa\Model\Geral\Repositories\ZhuAccessControlElement */
    protected $zhuAccessControlElementRepository;

    /** @var \Teknisa\Model\Geral\Repositories\ZhuAccessControlRule */
    protected $zhuAccessControlRuleRepository;

    /** @var \Teknisa\Model\Geral\Repositories\PerfAcessParc */
    protected $perfAcessParcRepository;

    /** @var \Teknisa\Model\Geral\Repositories\Operador */
    protected $operadorRepository;

    protected $projectId;
    /**
     * @var \Factory\ZhElement
     */
    private $zhElementFactory;
    /**
     * @var \Factory\ZhRule
     */
    private $zhRuleFactory;
    /**
     * @var \Zeedhi\ZhuAcl\Service\AccessControlRule
     */
    private $accessControlRule;
    /**
     * @var \Factory\PerfAcessParc
     */
    private $perfAcessParcFactory;

    const RULE_ALLOW = 'A';
    const RULE_READONLY = 'R';
    const RULE_INVISIBLE = 'I';

    protected $rowsIndexedByUniqueId = array();
    /**
     * @var \Teknisa\Libs\Service\AccessProfile
     */
    private $libAccessProfileService;

    public function __construct(EntityManager $entityManager,\Zeedhi\ZhuAcl\Service\AccessControlRule $accessControlRule,
                                \Factory\CpPerfacessoper $cpPerfacessoperFactory,\Factory\PerfAcessParc $perfAcessParcFactory,
                                \Factory\CpPerfilacesso $cpPerfilacessoFactory,\Factory\ZhElement $zhElementFactory,
                                \Factory\ZhRule $zhRuleFactory,$projectId,\Teknisa\Libs\Service\AccessProfile $libAccessProfileService) {

        $this->entityManager = $entityManager;
        $this->environment = Utilities::getEnvironment();
        $this->cpPerfacessoperRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\CpPerfacessoper');
        $this->perfAcessParcRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Perfacessparc');
        $this->operadorRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Operador');
        $this->cpPerfilacessorRepository = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\CpPerfilacesso');
        $this->zhuAccessControlElementRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\ZhuAccessControlElement');
        $this->zhuAccessControlRuleRepository = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\ZhuAccessControlRule');

        $this->cpPerfacessoperFactory = $cpPerfacessoperFactory;
        $this->cpPerfilacessoFactory = $cpPerfilacessoFactory;
        $this->projectId = $projectId;

        $this->zhElementFactory = $zhElementFactory;
        $this->zhRuleFactory = $zhRuleFactory;
        $this->accessControlRule = $accessControlRule;
        $this->perfAcessParcFactory = $perfAcessParcFactory;
        $this->libAccessProfileService = $libAccessProfileService;
    }

    public function saveAccessProfile($nrperfilacesso,$nmperfilacesso,$idativo,$idperfilacessoadm = 'N') {
        $cdCliente = $this->environment->getCurrentCdCliente();
        $perfilacesso = $this->cpPerfilacessorRepository->findOneBy(array('nrperfilacesso'=>$nrperfilacesso));
        $perfilacesso = $this->cpPerfilacessoFactory->save($perfilacesso,$nmperfilacesso,$cdCliente,$idativo,$idperfilacessoadm);
        $this->entityManager->persist($perfilacesso);
        $this->entityManager->flush();
        return $perfilacesso->getNrperfilacesso();
    }

    public function addPerfilInOperador($cdoperadors,$profileId) {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            if (Utilities::currentUserIsTeknisa()) {
                /** @var \Teknisa\Model\Geral\Entities\Operador[] $operadors */
                $operadors = $this->operadorRepository->findBy(array('cdoperador' => $cdoperadors));
                foreach ($operadors as $operador) {
                    $perfacessparc = $this->perfAcessParcRepository->findOneBy(
                        array('nrparcnegocio' => $operador->getNrparcnegocio(), 'nrperfilacesso' => $profileId));
                    if (!$perfacessparc) {
                        $perfacessparc = $this->perfAcessParcFactory->create($operador->getNrparcnegocio(), $profileId);
                        $this->entityManager->persist($perfacessparc);
                    }
                }
            } else {
                foreach ($cdoperadors as $cdoperador) {
                    $perfacessparc = $this->cpPerfacessoperRepository->findOneBy(
                        array('cdoperador' => $cdoperador, 'nrperfilacesso' => $profileId));
                    if (!$perfacessparc) {
                        $perfacessparc = $this->cpPerfacessoperFactory->create($cdoperador, $profileId);
                        $this->entityManager->persist($perfacessparc);
                    }
                }
            }
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function deletePerfilInOperador($cdoperadors,$profileId) {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            if (Utilities::currentUserIsTeknisa()) {
                /** @var \Teknisa\Model\Geral\Entities\Operador[] $operadors */
                $operadors = $this->operadorRepository->findBy(array('cdoperador' => $cdoperadors));
                foreach ($operadors as $operador) {
                    $perfacessparc = $this->perfAcessParcRepository->findOneBy(
                        array('nrparcnegocio' => $operador->getNrparcnegocio(), 'nrperfilacesso' => $profileId));
                    if ($perfacessparc) {
                        $this->entityManager->remove($perfacessparc);
                    }
                }
            } else {
                foreach ($cdoperadors as $cdoperador) {
                    $perfacessparc = $this->cpPerfacessoperRepository->findOneBy(
                        array('cdoperador' => $cdoperador, 'nrperfilacesso' => $profileId));
                    if ($perfacessparc) {
                       $this->entityManager->remove($perfacessparc);
                    }
                }
            }
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function createRowAclZeedhi($rule,$type,$uniqueId,$projectId,$name,$label,$groupId,$ruleId,$currentUserId){
        $containerRow = array();
        $containerRow['UNIQUE_ID'] = $uniqueId;
        $containerRow['USER_ID'] = $currentUserId;
        $containerRow['GROUP_ID'] = $groupId;
        $containerRow['NAME'] = $name;
        $containerRow['LABEL'] = $label;
        $containerRow['PROJECT_ID'] = $projectId;
        $containerRow['RULE'] = $rule;
        $containerRow['TYPE'] = $type;
        $containerRow['RULE_ID'] = $ruleId;
        $containerRow['ORGANIZATION_ID'] =  $this->environment->getCurrentOrganizationId();;

        return $containerRow;
    }

    public function saveAclZhuAclElementRules($rule,$type,$uniqueId,$projectId,$name,$label,$groupId,$ruleId,
                                              $containerUniqueId,$containerName,$containerLabel,$containerRuleId) {
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            $currentUserId = $this->environment->getCurrentUserId();
            $isContainer = ($containerUniqueId && $type == ZhuAclTableProperties::TYPE_MENU);
            if (Utilities::currentUserIsTeknisa()) {
                $zhRule = $this->saveRulesZeedhi($rule, $type, $uniqueId, $projectId, $name, $label, $groupId, $ruleId,
                    $currentUserId);
                $return = $zhRule->getId();
                if ($isContainer) {
                    $containerZhRule = $this->saveRulesZeedhi($rule, ZhuAclTableProperties::TYPE_CONTAINER, $containerUniqueId,
                        $projectId, $containerName, $containerLabel, $groupId, $containerRuleId, $currentUserId);
                    $return = array("menuId" => $zhRule->getId(),
                        "containerId" => $containerZhRule->getId());
                }
            } else {
                $zhRule = $this->saveRuleCP($rule, $type, $uniqueId, $projectId, $name, $label, $groupId, $currentUserId);
                $return = $zhRule->getId();
                if ($isContainer) {
                    $containerZhRule = $this->saveRuleCP($rule, ZhuAclTableProperties::TYPE_CONTAINER, $containerUniqueId,
                        $projectId, $containerName, $containerLabel, $groupId, $currentUserId);
                    $return = array("menuId" => $zhRule->getId(),
                        "containerId" => $containerZhRule->getId());
                }
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
        return $return;
    }

    public function findAccessControlRule($groupId) {
        $cdCliente = $this->environment->getCurrentCdCliente();
        $projectId = $this->projectId;
        $rules = $this->zhuAccessControlElementRepository->getRules($cdCliente,$projectId,$groupId);

        return $this->resolveGroupsConflicts($rules);
    }

    protected function resolveGroupsConflicts (array $rules) {
        $this->rowsIndexedByUniqueId = array();
        foreach($rules as $row) {
            $this->resolveRulesConflicts($row);
        }
        $rows = array();
        foreach($this->rowsIndexedByUniqueId as $row) {
            $rows[] = $row;
        }
        return $rows;
    }

    protected function resolveRulesConflicts($row) {
        $rowUniqueId = $row['UNIQUE_ID'];
        if (isset($this->rowsIndexedByUniqueId[$rowUniqueId])) {
            $this->rowsIndexedByUniqueId[$rowUniqueId]['RULE'] =
                $this->getMostPermissiveRule($this->rowsIndexedByUniqueId[$rowUniqueId]['RULE'], $row['RULE']);
        }
        else {
            $this->rowsIndexedByUniqueId[$rowUniqueId] = $row;
        }
    }

    /*
     * The order of permissive is ALLOW, READ ONLY, INVISIBLE
     */
    protected function getMostPermissiveRule($rule1, $rule2) {
        if ($rule1 == self::RULE_ALLOW || $rule2 == self::RULE_ALLOW) {
            return self::RULE_ALLOW;
        }
        else if ($rule1 == self::RULE_READONLY || $rule2 == self::RULE_READONLY) {
            return self::RULE_READONLY;
        }
        else {
            return self::RULE_INVISIBLE;
        }
    }

    public function findAclZhuAclElementRules($groupId) {
        $cdCliente = $this->environment->getCurrentCdCliente();
        $projectId = $this->projectId;
        $organizationId = $this->environment->getCurrentOrganizationId();
        $isteknisa = Utilities::currentUserIsTeknisa();
        return $this->zhuAccessControlElementRepository->getElements($organizationId,$cdCliente,$projectId,$groupId,$isteknisa);
    }

    public function getAccessProfile() {
        $organizationId = $this->environment->getCurrentOrganizationId();
        if(Utilities::currentUserIsTeknisa()){
            $accessProfiles = $this->libAccessProfileService->getAccessProfile($organizationId);
        }else{
            $cdcliente = $this->environment->getCurrentCdCliente();
            $accessProfiles = $this->cpPerfilacessorRepository->getAccessProfile($organizationId,$cdcliente);
        }
        return $accessProfiles;
    }

    /**
     * @param $rule
     * @param $type
     * @param $uniqueId
     * @param $projectId
     * @param $name
     * @param $label
     * @param $groupId
     * @param $ruleId
     * @param $currentUserId
     * @return \ZeedhiUser\Model\AccessControl\Entities\ZhuAccessControlRule
     */
    public function saveRulesZeedhi($rule, $type, $uniqueId, $projectId, $name, $label, $groupId, $ruleId, $currentUserId)
    {
        $row = $this->createRowAclZeedhi($rule, $type, $uniqueId, $projectId, $name, $label, $groupId,
            $ruleId, $currentUserId);
        $zhRule = $this->accessControlRule->insertRule($row);
        return $zhRule;
    }

    /**
     * @param $rule
     * @param $type
     * @param $uniqueId
     * @param $projectId
     * @param $name
     * @param $label
     * @param $groupId
     * @param $currentUserId
     * @return null|object|\Teknisa\Model\Geral\Entities\ZhuAccessControlRule
     */
    public function saveRuleCP($rule, $type, $uniqueId, $projectId, $name, $label, $groupId, $currentUserId)
    {
        $element = $this->zhuAccessControlElementRepository->findOneBy(array('uniqueId' => $uniqueId));
        $element = $this->zhElementFactory->save($element, $type, $uniqueId, $projectId, $name, $label, $currentUserId);
        $this->entityManager->persist($element);
        $this->entityManager->flush();
        /** @var \Teknisa\Model\Geral\Entities\ZhuAccessControlRule $rule */
        $zhRule = $this->zhuAccessControlRuleRepository->findOneBy(
            array('accessControlElementId' => $element->getId(), 'groupId' => $groupId));
        $zhRule = $this->zhRuleFactory->save($zhRule, $rule, $element, $groupId, $currentUserId);
        $this->entityManager->persist($zhRule);
        $this->entityManager->flush();
        return $zhRule;
    }
} 